 /*
 *
 *  ==  Coconut ==
 *  2D Adventure Game Engine
 *
 *  Copyright (C) 2000 Sebastien Josset
 *  sebastien.josset@free.fr
 *
 *  The originals functions are part of the SGE Library
 *  which is under LGPL v2+ License
 *	(c)1999,2000 Anders Lindstrm
 *  Anders Lindstrm - cal@swipnet.se
 *  Little modifications have been done for the Coconut use : 
 *  e.g. Copy the transparency color when rescaling.
 */

/*********************************************************************
 *  This library is free software; you can redistribute it and/or    *
 *  modify it under the terms of the GNU Library General Public      *
 *  License as published by the Free Software Foundation; either     *
 *  version 2 of the License, or (at your option) any later version. *
 *********************************************************************/



#include <math.h>
#include "SDL.h"
#include <memory.h>


//==================================================================================
// SDL Graphics Extension (SGE) - http://www.etek.chalmers.se/~e8cal1/sge/index.html
//==================================================================================
//==================================================================================
// Clear surface to color
//==================================================================================

void sge_ClearSurface(SDL_Surface *Surface, Uint32 color)
{

	SDL_FillRect(Surface,NULL, color);

	//if(_sge_update!=1){return;}
	SDL_UpdateRect(Surface, 0,0,0,0);
}

void sge_write_block8(SDL_Surface *Surface, Uint8 *block, Sint32 y)
{
	memcpy(	(Uint8 *)Surface->pixels + y*Surface->pitch, block, sizeof(Uint8)*Surface->w );
}
void sge_write_block16(SDL_Surface *Surface, Uint16 *block, Sint32 y)
{
	memcpy(	(Uint16 *)Surface->pixels + y*Surface->pitch/2, block, sizeof(Uint16)*Surface->w );
}
void sge_write_block32(SDL_Surface *Surface, Uint32 *block, Sint32 y)
{
	memcpy(	(Uint32 *)Surface->pixels + y*Surface->pitch/4, block, sizeof(Uint32)*Surface->w );
}


Uint32 sge_GetPixel(SDL_Surface *surface, Sint32 x, Sint32 y)
{
	switch (surface->format->BytesPerPixel) {
			case 1: { /* Assuming 8-bpp */
				return *((Uint8 *)surface->pixels + y*surface->pitch + x);
			}
			break;

			case 2: { /* Probably 15-bpp or 16-bpp */
				return *((Uint16 *)surface->pixels + y*surface->pitch/2 + x);
			}
			break;

			case 3: { /* Slow 24-bpp mode, usually not used */
				Uint8 *pix;
				int shift;
				Uint32 color=0;

				/* Does this work? */
				pix = (Uint8 *)surface->pixels + y * surface->pitch + x*3;
				shift = surface->format->Rshift;
				color = *(pix+shift/8)>>shift;
				shift = surface->format->Gshift;
				color|= *(pix+shift/8)>>shift;
				shift = surface->format->Bshift;
				color|= *(pix+shift/8)>>shift;
				return color;
			}
			break;

			case 4: { /* Probably 32-bpp */
				return *((Uint32 *)surface->pixels + y*surface->pitch/4 + x);
			}
			break;
		}
	return 0;
}






//==================================================================================
// Fast put pixel
//==================================================================================
void _PutPixel(SDL_Surface *surface, Sint32 x, Sint32 y, Uint32 color)
{

	if(x>=0 && x<=640 && y>=0 && y<=480){
		switch (surface->format->BytesPerPixel) {
			case 1: { /* Assuming 8-bpp */
				*((Uint8 *)surface->pixels + y*surface->pitch + x) = color;
			}
			break;

			case 2: { /* Probably 15-bpp or 16-bpp */
				*((Uint16 *)surface->pixels + y*surface->pitch/2 + x) = color;
			}
			break;

			case 3: { /* Slow 24-bpp mode, usually not used */
				Uint8 *pix;
  			int shift;

  			/* Gack - slow, but endian correct */
  			pix = (Uint8 *)surface->pixels + y * surface->pitch + x*3;
  			shift = surface->format->Rshift;
  			*(pix+shift/8) = color>>shift;
  			shift = surface->format->Gshift;
  			*(pix+shift/8) = color>>shift;
  			shift = surface->format->Bshift;
  			*(pix+shift/8) = color>>shift;
			}
			break;

			case 4: { /* Probably 32-bpp */
				*((Uint32 *)surface->pixels + y*surface->pitch/4 + x) = color;
			}
			break;
		}
	}

}

//==================================================================================
// Internal draw hor. line
//==================================================================================
void _HLine(SDL_Surface *Surface, Sint32 x1, Sint32 x2, Sint32 y, Uint32 Color)
{
	if(x1>x2){Sint32 tmp=x1; x1=x2; x2=tmp;}

   /* Draw the line */
   for (Sint32 i = x1; i <= x2; i++)
      _PutPixel(Surface, i, y, Color);
}

//==================================================================================
// Internal draw ver. line
//==================================================================================
void _VLine(SDL_Surface *Surface, Sint32 x, Sint32 y1, Sint32 y2, Uint32 Color)
{
	if(y1>y2){Sint32 tmp=y1; y1=y2; y2=tmp;}

   /* Draw the line */
   for (Sint32 i = y1; i <= y2; i++)
      _PutPixel(Surface, x, i, Color);
}



//==================================================================================
// Draws a rectangle
//==================================================================================
void sge_Rect(SDL_Surface *Surface, Sint32 x1, Sint32 y1, Sint32 x2, Sint32 y2, Uint32 color)
{
	//Uint32 color = SDL_MapRGB(Surface->format, R, G, B);
	
	_HLine(Surface,x1,x2,y1,color);
	_HLine(Surface,x1,x2,y2,color);
	_VLine(Surface,x1,y1,y2,color);
	_VLine(Surface,x2,y1,y2,color);

	/*
	sge_UpdateRect(Surface, x1, y1, x2-x1, 1);
	sge_UpdateRect(Surface, x1, y2, x2-x1+1, 1); 
	sge_UpdateRect(Surface, x1, y1, 1, y2-y1);
	sge_UpdateRect(Surface, x2, y1, 1, y2-y1);
	*/
}









SDL_Surface *sge_rotate_scaled_surface(SDL_Surface *src, int angle, float scale, Uint32 bcol)
{
	SDL_Surface *dest;

	/* Create the destination surface*/
	int max=(int)((src->h + src->w)*scale);
	dest=SDL_AllocSurface(SDL_SWSURFACE, max, max, src->format->BitsPerPixel, src->format->Rmask, src->format->Gmask, src->format->Bmask, src->format->Amask );
	if(!dest){SDL_SetError("SGE - Out of memory");return NULL;}

	// Si le fond est transparent, on le remap
	if ((src->flags)&SDL_SRCCOLORKEY) {
		bcol = src->format->colorkey;
		SDL_SetColorKey(dest, SDL_SRCCOLORKEY, bcol); 
	}

	sge_ClearSurface(dest,bcol);	


	float rad=(float)angle*0.017453292;  //Deg => rad

 	float f1x, f1y, f1xd, f1yd;
 	float f2x, f2y, f2xd, f2yd;
 	int w, h;
 	float dir, dist;
 	int dx, dy;
 	int sx, sy;
 	int wgap = src->w;
 	int hgap = src->h;

 	/* rotate the top left pixel of the sprite */
 	w = wgap>>1;///2.0;
 	h = hgap>>1;///2.0;
 	dir = atan2(h, w)+0.43*log(float(src->w)/float(src->h)); //...+0.43*log...: Don't ask, it works, ok?
 	dist = sqrt((wgap*wgap + hgap*hgap));
 	f1x = w - dist*cos(dir - rad);
 	f1y = h - dist*sin(dir - rad);

 	/* map the destination y axis onto the sprite */
 	f1xd = cos(1.570796 - rad);
 	f1yd = sin(1.570796 - rad);

 	/* map the destination x axis onto the sprite */
 	f2xd = cos(-rad);
 	f2yd = sin(-rad);

 	/* scale the sprite? */
 	if (scale != 1.0) {
  	dist = dist*scale;
   	wgap = (int)(wgap*scale);
   	hgap = (int)(hgap*scale);
   	scale = 1.0/scale;
  	f1xd = f1xd*scale;
   	f1yd = f1yd*scale;
   	f2xd = f2xd*scale;
   	f2yd = f2yd*scale;
 	}

 	/* trace a bunch of lines through the bitmaps */
	for (dy=0; dy<dest->h; dy++) {
  	f2x = f1x;
   	f2y = f1y;

 		/* draw onto a linear bitmap */
	 	switch (src->format->BytesPerPixel) {

	  	case 1:{
	    	/* 8 bit version */
				unsigned char block8[1025];

	    	for (dx=0; dx<dest->w; dx++) {
		  		sx = (int)f2x;
		  		sy = (int)f2y;

		  		if ((sx >= 0) && (sx < src->w) && (sy >= 0) && (sy < src->h) ){
          	block8[dx]= (Uint8)sge_GetPixel(src, sx, sy);
					}
					else{block8[dx]=(Uint8)bcol;} //outside source

		  		f2x += f2xd;
		  		f2y += f2yd;
	   		}
     		sge_write_block8(dest, block8, dy); //copy the new block to destination
			}	
	   	break;

	   	case 2:{
		  	/* 16 bit version */
       	unsigned short block16[1025];

		  	for (dx=0; dx<dest->w; dx++) {
		    	sx = (int)f2x;
		     	sy = (int)f2y;

		    	if ((sx >= 0) && (sx < src->w) && (sy >= 0) && (sy < src->h) ) {
						block16[dx]= (Uint16)sge_GetPixel(src, sx, sy);	
		     	}
					else{block16[dx]=(Uint16)bcol;} //outside source

		     	f2x += f2xd;
		     	f2y += f2yd;
		  	}
       	sge_write_block16(dest, block16, dy); //copy the new block to destination
	    }
		 	break;

	   	case 3:{
		  	/* 24 bit version */
      	//I hate 24 bit...
			}
		 	break;

	  	case 4:{
		  	/* 32 bit version */
				Uint32 block32[1025];

		  	for (dx=0; dx<dest->w; dx++) {
		    	sx = (int)f2x;
		     	sy = (int)f2y;

		    	if ((sx >= 0) && (sx < src->w) && (sy >= 0) && (sy < src->h) ) {
          	block32[dx]= sge_GetPixel(src, sx, sy);
		     	}
					else{block32[dx]=bcol;} //outside source

		     	f2x += f2xd;
		     	f2y += f2yd;
		 		}
       	sge_write_block32(dest, block32, dy); //copy the new block to destination
     	}
			break;

		}
		f1x += f1xd;
   	f1y += f1yd;
 	}


	return(dest);
}

