/*
 *
 *  ==  Coconut ==
 *  2D Adventure Game Engine
 *
 *  Copyright (C) 2000 Sebastien Josset
 *  sebastien.josset@free.fr
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 */

/*
 *
 *  Coconut
 *  Objets
 *
 *
 */

/****** Includes systmes ******/
#include <math.h>
#include <string.h>

/****** Includes applicatifs ******/
#include "O_objets.h"
#include "O_panneau.h"
#include "R_ressources.h"
#include "M_scenes.h"
#include "M_files.h"
#include "M_engine.h"
#include "I_input.h"
#include "A_aff.h"


TC_Object O_gTab_Objets[Nb_objetsMax];
int		  O_gNb_objets=0;
char	  O_gSelected_Name[NAMELEN]="";


int cgb_obj_set_selected_name(char *ip_name)
{
	if (ip_name) {
		strncpy(O_gSelected_Name, ip_name, NAMELEN);
	}
	return 0;
}

char *cgb_obj_get_selected_name()
{
	return O_gSelected_Name;
}

int cgb_obj_is_selected_name(char *ip_name)
{
	int l_ret;

	if (!ip_name) { return 0; }
	if (strcmp(O_gSelected_Name, ip_name)==0) {
		l_ret = 1;
	} else {
		l_ret =0;
	}
	return l_ret;
}


int  cgb_obj_clear()
{
	int l_cpt;

	for (l_cpt=0; l_cpt<Nb_objetsMax; l_cpt++) {
		O_gTab_Objets[l_cpt].Clear();
	}
	O_gNb_objets=0;
	return 0;
}

void cgb_obj_del(char *ip_name)
{
    int l_index;
    int l_cpt;

    l_index = cgb_obj_get_index_by_name(ip_name);
    if (l_index==0)  { return; }
    for (l_cpt=l_index; l_cpt<(O_gNb_objets-1); l_cpt++) {
        O_gTab_Objets[l_cpt] = O_gTab_Objets[l_cpt+1];
    }
    O_gNb_objets--;
}

TPC_Object cgb_obj_get_ptr_by_name(char *ip_name)
{
	int l_cpt;
	int l_obj;

	l_obj=0; // Objet null
	for (l_cpt=0; l_cpt<O_gNb_objets; l_cpt++) {
		if (O_gTab_Objets[l_cpt].isName(ip_name)) {
			l_obj = l_cpt;
			l_cpt = O_gNb_objets;
		}
	}
	return &(O_gTab_Objets[l_obj]);
}

TPC_Object cgb_obj_get_ptr_by_index(int i_index)
{
	return &(O_gTab_Objets[i_index]);
}

int cgb_obj_get_count()
{
	return O_gNb_objets;
}


int cgb_obj_get_index_by_name(char *ip_name)
{
	int l_cpt;
	int l_obj;

	l_obj=0; // Objet null
	for (l_cpt=0; l_cpt<O_gNb_objets; l_cpt++) {
		if (O_gTab_Objets[l_cpt].isName(ip_name)) {
			l_obj = l_cpt;
			l_cpt = O_gNb_objets;
		}
	}
	return l_obj;
}

int cgb_obj_add_attr(
			   char *ip_bufName,
			   char *ip_bufAffName,
			   char *ip_bufResAnimName,
			   char *ip_bufSceneName,
			   int i_ptCoord_x, int i_ptCoord_y,
			   int i_ptSize_x, int i_ptSize_y,
			   int i_ptHotSpot_x, int i_ptHotSpot_y,
			   int l_ptText_x, int l_ptText_y,
			   T_BOOL i_isClickable,
			   T_BOOL i_isAfficher, T_BOOL i_isZoneDep, T_BOOL i_isDragAndDrop,
			   char *ip_bufSceneSuivanteName,
			   char *ip_bufSceneSuivantePoint)
{
    if (O_gNb_objets<=0) { O_gNb_objets = 1; }
	O_gTab_Objets[O_gNb_objets].SetName(ip_bufName);
	O_gTab_Objets[O_gNb_objets].SetAffName(ip_bufAffName);
	O_gTab_Objets[O_gNb_objets].SetResAnimName(ip_bufResAnimName);
	O_gTab_Objets[O_gNb_objets].SetSceneName(ip_bufSceneName);
	O_gTab_Objets[O_gNb_objets].SetSceneSuivanteName(ip_bufSceneSuivanteName);
	O_gTab_Objets[O_gNb_objets].SetSceneSuivantePoint(ip_bufSceneSuivantePoint);
	O_gTab_Objets[O_gNb_objets].Scene = (TE_SceneId)cgb_scenes_get_index_by_name(ip_bufSceneName);
	O_gTab_Objets[O_gNb_objets].SetptCoordInit(i_ptCoord_x, i_ptCoord_y);
	O_gTab_Objets[O_gNb_objets].SetptCoord(i_ptCoord_x, i_ptCoord_y);
	O_gTab_Objets[O_gNb_objets].SetptSize(i_ptSize_x, i_ptSize_y);
	O_gTab_Objets[O_gNb_objets].SetptText(l_ptText_x, l_ptText_y);
	O_gTab_Objets[O_gNb_objets].SetptHotSpot(i_ptHotSpot_x, i_ptHotSpot_y);
	O_gTab_Objets[O_gNb_objets].SetisClickable(i_isClickable);
    O_gTab_Objets[O_gNb_objets].SetisAfficher(i_isAfficher);
    O_gTab_Objets[O_gNb_objets].SetisZoneDep(i_isZoneDep);
    O_gTab_Objets[O_gNb_objets].SetisDragAndDrop(i_isDragAndDrop);
    O_gNb_objets++;
	return 0;
}

int cgb_obj_add_default(char *ip_bufName)
{
    if (O_gNb_objets<=0) { O_gNb_objets = 1; }
	O_gTab_Objets[O_gNb_objets].SetName(ip_bufName);
	O_gNb_objets++;
	return 0;
}



int cgb_obj_ren(char *ip_name_old, char *ip_name_new)
{
	int l_cpt;
	l_cpt = cgb_obj_get_index_by_name(ip_name_old);
	O_gTab_Objets[l_cpt].SetName(ip_name_new);
	return 0;
}

/*
int  cgb_obj_add_from_file(char *ip_bufObjet, char *ip_fileName)
{
	int  l_ret;
	char l_bufName[NAMELEN];
	char l_bufResAnimName[NAMELEN];
	char l_bufSceneName[NAMELEN];
	char l_bufSceneSuivanteName[NAMELEN];
	char l_bufSceneSuivantePoint[NAMELEN];
	int  l_ptCoord_x;
	int  l_ptCoord_y;
	int  l_ptText_x;
	int  l_ptText_y;
	int  l_ptSize_x;
	int  l_ptSize_y;
	int  l_ptHotSpot_x;
	int  l_ptHotSpot_y;
	int  l_isClickable;
	int  l_isAfficher;
	int  l_isZoneDep;
	int  l_isDragAndDrop;

	l_ret = GetPrivateProfileString(ip_bufObjet, "Name", "", l_bufName, NAMELEN, ip_fileName);
	l_ret = GetPrivateProfileString(ip_bufObjet, "ResAnimName", "", l_bufResAnimName, NAMELEN, ip_fileName);
	l_ret = GetPrivateProfileString(ip_bufObjet, "SceneName", "", l_bufSceneName, NAMELEN, ip_fileName);
	l_ret = GetPrivateProfileString(ip_bufObjet, "SceneSuivanteName", "", l_bufSceneSuivanteName, NAMELEN, ip_fileName);
	l_ret = GetPrivateProfileString(ip_bufObjet, "SceneSuivantePoint", "", l_bufSceneSuivantePoint, NAMELEN, ip_fileName);
	l_ptCoord_x = GetPrivateProfileInt   (ip_bufObjet, "ptCoord.x", 0, ip_fileName);
	l_ptCoord_y = GetPrivateProfileInt   (ip_bufObjet, "ptCoord.y", 0, ip_fileName);
	l_ptSize_x = GetPrivateProfileInt   (ip_bufObjet, "ptSize.x", 0, ip_fileName);
	l_ptSize_y = GetPrivateProfileInt   (ip_bufObjet, "ptSize.y", 0, ip_fileName);
	l_ptHotSpot_x = GetPrivateProfileInt   (ip_bufObjet, "ptHotSpot.x", 0, ip_fileName);
	l_ptHotSpot_y = GetPrivateProfileInt   (ip_bufObjet, "ptHotSpot.y", 0, ip_fileName);
	l_ptText_x = GetPrivateProfileInt   (ip_bufObjet, "ptText.x", 0, ip_fileName);
	l_ptText_y = GetPrivateProfileInt   (ip_bufObjet, "ptText.y", 0, ip_fileName);
	l_isClickable = GetPrivateProfileInt   (ip_bufObjet, "isClickable", 0, ip_fileName);
	l_isAfficher = GetPrivateProfileInt   (ip_bufObjet, "isAfficher", 1, ip_fileName);
	l_isZoneDep  = GetPrivateProfileInt   (ip_bufObjet, "isZoneDep", 0, ip_fileName);
	l_isDragAndDrop = GetPrivateProfileInt(ip_bufObjet, "isDragAndDrop", 0, ip_fileName);

	cgb_obj_add_attr(l_bufName, l_bufResAnimName, l_bufSceneName,
		l_ptCoord_x, l_ptCoord_y,
		l_ptSize_x, l_ptSize_y,
		l_ptHotSpot_x, l_ptHotSpot_y,
		l_ptText_x, l_ptText_y,
		l_isClickable,
		l_isAfficher, l_isZoneDep, l_isDragAndDrop,
		l_bufSceneSuivanteName, l_bufSceneSuivantePoint
		);
	return 0;

}
*/

int  cgb_obj_add_from_mem(char *ip_bufObjet)
{
	int  l_ret;
	char l_bufName[NAMELEN];
	char l_bufAffName[NAMELEN];
	char l_bufResAnimName[NAMELEN];
	char l_bufSceneName[NAMELEN];
	char l_bufSceneSuivanteName[NAMELEN];
	char l_bufSceneSuivantePoint[NAMELEN];
	int  l_ptCoord_x;
	int  l_ptCoord_y;
	int  l_ptText_x;
	int  l_ptText_y;
	int  l_ptSize_x;
	int  l_ptSize_y;
	int  l_ptHotSpot_x;
	int  l_ptHotSpot_y;
	int  l_isClickable;
	int  l_isAfficher;
	int  l_isZoneDep;
	int  l_isDragAndDrop;

	l_ret = M_getParamStr(ip_bufObjet, "Name", "", l_bufName, NAMELEN);
	l_ret = M_getParamStr(ip_bufObjet, "AffName", "", l_bufAffName, NAMELEN);
	l_ret = M_getParamStr(ip_bufObjet, "ResAnimName", "", l_bufResAnimName, NAMELEN);
	l_ret = M_getParamStr(ip_bufObjet, "SceneName", "", l_bufSceneName, NAMELEN);
	l_ret = M_getParamStr(ip_bufObjet, "SceneSuivanteName", "", l_bufSceneSuivanteName, NAMELEN);
	l_ret = M_getParamStr(ip_bufObjet, "SceneSuivantePoint", "", l_bufSceneSuivantePoint, NAMELEN);
	l_ptCoord_x = M_getParamInt   (ip_bufObjet, "ptCoord.x", 0);
	l_ptCoord_y = M_getParamInt   (ip_bufObjet, "ptCoord.y", 0);
	l_ptSize_x = M_getParamInt   (ip_bufObjet, "ptSize.x", 0);
	l_ptSize_y = M_getParamInt   (ip_bufObjet, "ptSize.y", 0);
	l_ptHotSpot_x = M_getParamInt   (ip_bufObjet, "ptHotSpot.x", 0);
	l_ptHotSpot_y = M_getParamInt   (ip_bufObjet, "ptHotSpot.y", 0);
	l_ptText_x = M_getParamInt   (ip_bufObjet, "ptText.x", 0);
	l_ptText_y = M_getParamInt   (ip_bufObjet, "ptText.y", 0);
	if (l_ptText_y==0) l_ptText_y=-20;
	l_isClickable = M_getParamInt   (ip_bufObjet, "isClickable", 0);
	l_isAfficher = M_getParamInt   (ip_bufObjet, "isAfficher", 1);
	l_isZoneDep  = M_getParamInt   (ip_bufObjet, "isZoneDep", 0);
	l_isDragAndDrop = M_getParamInt(ip_bufObjet, "isDragAndDrop", 0);

	cgb_obj_add_attr(l_bufName, l_bufAffName, l_bufResAnimName, l_bufSceneName,
		l_ptCoord_x, l_ptCoord_y,
		l_ptSize_x, l_ptSize_y,
		l_ptHotSpot_x, l_ptHotSpot_y,
		l_ptText_x, l_ptText_y,
		l_isClickable,
		l_isAfficher, l_isZoneDep, l_isDragAndDrop,
		l_bufSceneSuivanteName, l_bufSceneSuivantePoint
		);
	return 0;

}


int TC_Object::Clear()
{
	SetName("");
	SetResAnimName("");
	SetSceneName("");
	SetSceneSuivanteName("");
	SetptCoord(0, 0);
	SetptCoordInit(0, 0);
	SetptSize(0, 0);
	SetptHotSpot(0, 0);
	SetisClickable(T_FALSE);
	SetScale(0);
	return 0;
}


int TC_Object::SetName(char *ip_name)
{
	strcpy(name, ip_name);
	return 0;
}

int TC_Object::SetAffName(char *ip_name)
{
	strcpy(affName, ip_name);
	return 0;
}

int TC_Object::getWidth()
{
	if (PtImageAnimCourante) {
		return (PtImageAnimCourante->recRect.right - PtImageAnimCourante->recRect.left);
	} else  {
		return 0;
	}
}

int TC_Object::SetScale(float i_scale)
{
	scale = i_scale;
	return 0;
}

int TC_Object::SetResAnimName(char *ip_name)
{
	strcpy(ResAnimName, ip_name);
	ptAnimCourante = NULL;
	return 0;
}

int TC_Object::SetSceneName(char *ip_name)
{
	strcpy(SceneName, ip_name);
	return 0;
}

int TC_Object::SetSceneSuivanteName(char *ip_name)
{
	strcpy(SceneSuivanteName, ip_name);
	return 0;
}

int TC_Object::SetSceneSuivantePoint(char *ip_name)
{
	strcpy(SceneSuivantePoint, ip_name);
	return 0;
}


T_BOOL TC_Object::isName(char *ip_name)
{
    T_BOOL l_ret;
    if (strcmp(name, ip_name)==0) {
        l_ret = T_TRUE;
    } else {
        l_ret = T_FALSE;
    }
    return l_ret;
}


T_BOOL TC_Object::isSceneName(char *ip_name)
{
    T_BOOL l_ret;
    if (strcmp(SceneName, ip_name)==0) {
        l_ret = T_TRUE;
    } else {
        l_ret = T_FALSE;
    }
    return l_ret;
}

T_BOOL TC_Object::isResAnimName(char *ip_name)
{
    T_BOOL l_ret;
    if (strcmp(ResAnimName, ip_name)==0) {
        l_ret = T_TRUE;
    } else {
        l_ret = T_FALSE;
    }
    return l_ret;
}

T_BOOL TC_Object::isChgtScene()
{
    T_BOOL l_ret;
    if (strlen(SceneSuivanteName)==0) {
        l_ret = T_FALSE;
    } else {
        l_ret = T_TRUE;
    }
    return l_ret;
}

T_BOOL TC_Object::isZoneDep()
{
    return is_ZoneDep;
}


int TC_Object::SetptCoord(int i_x, int i_y)
{
	ptCoord.x = i_x;
	ptCoord.y = i_y;
	return 0;
}

void TC_Object::Reinit()
{
	ptCoord.x = ptCoordInit.x;
	ptCoord.y = ptCoordInit.y;
}


int TC_Object::SetptCoordInit(int i_x, int i_y)
{
	ptCoordInit.x = i_x;
	ptCoordInit.y = i_y;
	return 0;
}



int TC_Object::SetptText(int i_x, int i_y)
{
	ptText.x = i_x;
	ptText.y = i_y;
	return 0;
}


int TC_Object::AffText(char *ip_resName, T_DWORD i_dureeAff, T_BOOL i_isFond)
{
//	if ((ptText.x!=0)||(ptText.y!=0)||(ptAnimCourante==NULL)) {
		idText = cgb_text_set(ip_resName, ptCoord.x+ptText.x, ptCoord.y+ptText.y, i_dureeAff, idText, i_isFond);
//	} else {
//		idText = cgb_text_set(ip_resName, ptCoord.x+ptAnimCourante->recRect.right, ptCoord.y, i_dureeAff, idText, i_isFond);
//	}
	return idText;
}

void TC_Object::SetTextScroll(int i_nbLignes, T_DWORD i_timerScroll)
{
	cgb_text_scroll_set(idText, i_nbLignes, i_timerScroll);	
}

int TC_Object::SetptSize(int i_x, int i_y)
{
	ptSize.x = i_x;
	ptSize.y = i_y;
	return 0;
}

int TC_Object::SetptHotSpot(int i_x, int i_y)
{
	ptHotSpot.x = i_x;
	ptHotSpot.y = i_y;
	return 0;
}

int TC_Object::SetisClickable(T_BOOL i_val)
{
	isClickable = i_val;
	return 0;
}

int TC_Object::SetisAfficher(T_BOOL i_val)
{
	isAfficher = i_val;
	return 0;
}

int TC_Object::SetisZoneDep(T_BOOL i_val)
{
	is_ZoneDep = i_val;
	return 0;
}

int TC_Object::SetisDragAndDrop(T_BOOL i_val)
{
	isDragAndDrop = i_val;
	return 0;
}


void TC_Object::SetAnimCourante(TE_ResId i_idResAnim)
{
	if (i_idResAnim==0) {
		ptAnimCourante =  NULL;
		PtImageAnimCourante = NULL;
	} else {
		ptAnimCourante = &(R_gTab_res[i_idResAnim].resAnim);
		PtImageAnimCourante = ptAnimCourante;
		strcpy(ResAnimName, R_gTab_res[i_idResAnim].name);
	}
}

void TC_Object::SetAnimCourantebyName(char *ip_name)
{
	TE_ResId l_idResAnim;
	l_idResAnim = cgb_res_anim_get_index_by_name(ip_name);
	SetAnimCourante(l_idResAnim);
}


void TC_Object::SetPointDest(char *ip_pointDest, long i_tempsParcours)
{
	TC_Object *l_obj;

	l_obj = cgb_obj_get_ptr_by_name(ip_pointDest);
	if (l_obj) {
		ptCoordDest.x = l_obj->ptCoord.x;
		ptCoordDest.y = l_obj->ptCoord.y;
		timeDest = s_LastTickCount+i_tempsParcours;
	}
}

void TC_Object::SetPointDest(char *ip_pointDest, int i_dx, int i_dy, long i_tempsParcours)
{
	TC_Object *l_obj;

	l_obj = cgb_obj_get_ptr_by_name(ip_pointDest);
	if (l_obj) {
		ptCoordDest.x = l_obj->ptCoord.x +i_dx;
		ptCoordDest.y = l_obj->ptCoord.y +i_dy;
		timeDest = s_LastTickCount+i_tempsParcours;
	}
}


int TC_Object::Anim()
{
	T_DWORD	l_tickCount;
	int		l_ret=0;
	float	l_dx;
	float	l_dy;

	// Deplacement eventuel
	// On doit aller quelque part
	if (timeDest>s_LastTickCount) {
		l_dx = ((float)(ptCoordDest.x - (ptCoord.x+ptHotSpot.x)))/((float)(timeDest-s_LastTickCount))*((float)s_DureeAff);
		l_dy = ((float)(ptCoordDest.y - (ptCoord.y+ptHotSpot.y)))/((float)(timeDest-s_LastTickCount))*((float)s_DureeAff);
		ptCoordResteX+=l_dx;
		ptCoordResteY+=l_dy;
		ptCoord.x += (long)floor(ptCoordResteX);
		ptCoord.y += (long)floor(ptCoordResteY);
		ptCoordResteX -= (float)floor(ptCoordResteX);
		ptCoordResteY -= (float)floor(ptCoordResteY);
	} else {
		// On a de la vitesse
		if (ptVit.x) { ptCoord.x += ptVit.x; }
		if (ptVit.y) { ptCoord.y += ptVit.y; }
	}

	// Scrolling eventuel
	if (ptVitScroll.x) { ptScroll.x += ptVitScroll.x; }
	if (ptVitScroll.y) { ptScroll.y += ptVitScroll.y; }

	l_tickCount = I_GetTickCount();
	// Si il y a une anim est cours, on passe  l'image suivante
	if (PtImageAnimCourante) {
		if (cptTick<l_tickCount) {
			PtImageAnimCourante = PtImageAnimCourante->pNext;
			l_ret=1;
			if (PtImageAnimCourante) {
				cptTick = l_tickCount + PtImageAnimCourante->tempo;
			} else {
				cptTick = 0;
			}
		}
	}
	// PtImageAnimCourante=NULL
	//		-> on est  la fin de l'anim (on reboucle)
	//		-> Mais ptAnimCourante peut tre null entre-temps !
	if (!PtImageAnimCourante) {
		PtImageAnimCourante = ptAnimCourante;
		if (PtImageAnimCourante) {
			cptTick = l_tickCount + PtImageAnimCourante->tempo;
		} else {
			cptTick = 0;
		}
	}
	return l_ret;
}

void TC_Object::AffOffScreen(TP_DDSurface ip_Dest)
{
	int l_res;
	// Si y a pas d'anim, mais qu'il y a une ressource associe, 
	// on la charge
	if (!PtImageAnimCourante) {
		if (ResAnimName[0]!=0) {
			l_res = cgb_res_anim_get_index_by_name(ResAnimName);
			SetAnimCourante((TE_ResId)l_res);
		}
	}

	if (PtImageAnimCourante) {
		if (M_g_coco_editor) {
			A_gAff.traceEmptyRect(ptCoord.x,  ptCoord.y, ptCoord.x+PtImageAnimCourante->recRect.right,  ptCoord.y+PtImageAnimCourante->recRect.bottom, 0);
			if (cgb_obj_is_selected_name(name)) {
				A_gAff.traceEmptyRect(ptCoord.x-1,  ptCoord.y-1, ptCoord.x+PtImageAnimCourante->recRect.right+1,  ptCoord.y+PtImageAnimCourante->recRect.bottom+1, 0);
			}
		}
		PtImageAnimCourante->AffOffScreen(ip_Dest, ptCoord, scale, ptScroll.x, ptScroll.y);
	} else {
		if (M_g_coco_editor) {
			A_gAff.traceEmptyRect(ptCoord.x,  ptCoord.y, ptCoord.x+ptSize.x,  ptCoord.y+ptSize.y, 0);
			if (cgb_obj_is_selected_name(name)) {
				A_gAff.traceEmptyRect(ptCoord.x-1,  ptCoord.y-1, ptCoord.x+ptSize.x+1,  ptCoord.y+ptSize.y+1, 0);
			}
		}
	}
	if (M_g_coco_editor) {
		A_gAff.traceEmptyRect(
			ptCoord.x+ptHotSpot.x-3,  
			ptCoord.y+ptHotSpot.y-3, 
			ptCoord.x+ptHotSpot.x+3,  
			ptCoord.y+ptHotSpot.y+3, 
			0);
		if (cgb_obj_is_selected_name(name)) {
			A_gAff.traceEmptyRect(
			ptCoord.x+ptHotSpot.x-4,  
			ptCoord.y+ptHotSpot.y-4, 
			ptCoord.x+ptHotSpot.x+4,  
			ptCoord.y+ptHotSpot.y+4, 
			0);
		}
	}

}

void TC_Object::AffOffScreen(TP_DDSurface ip_Dest, long i_dX, long i_dY)
{
	int l_res;
	// Si y a pas d'anim, mais qu'il y a une ressource associe, 
	// on la charge
	if (!PtImageAnimCourante) {
		if (ResAnimName[0]!=0) {
			l_res = cgb_res_anim_get_index_by_name(ResAnimName);
			SetAnimCourante((TE_ResId)l_res);
		}
	}
	
	if (PtImageAnimCourante) {
		if (M_g_coco_editor) { // Draws rect
			A_gAff.traceEmptyRect(ptCoord.x,  ptCoord.y, ptCoord.x+PtImageAnimCourante->recRect.right,  ptCoord.y+PtImageAnimCourante->recRect.bottom, 0);
			if (cgb_obj_is_selected_name(name)) {
				A_gAff.traceEmptyRect(ptCoord.x-1,  ptCoord.y-1, ptCoord.x+PtImageAnimCourante->recRect.right+1,  ptCoord.y+PtImageAnimCourante->recRect.bottom+1, 0);
			}
		}
		PtImageAnimCourante->AffOffScreen(ip_Dest, ptCoord, scale, ptScroll.x, ptScroll.y);
	} else {
		if (M_g_coco_editor) {
			A_gAff.traceEmptyRect(ptCoord.x,  ptCoord.y, ptCoord.x+ptSize.x,  ptCoord.y+ptSize.y, 0);
			if (cgb_obj_is_selected_name(name)) {
				A_gAff.traceEmptyRect(ptCoord.x-1,  ptCoord.y-1, ptCoord.x+ptSize.x+1,  ptCoord.y+ptSize.y+1, 0);
			}
		}
	}
	if (M_g_coco_editor) { // Draw hotspot
		A_gAff.traceEmptyRect(
			ptCoord.x+ptHotSpot.x-3,  
			ptCoord.y+ptHotSpot.y-3, 
			ptCoord.x+ptHotSpot.x+3,  
			ptCoord.y+ptHotSpot.y+3, 
			0);
		if (cgb_obj_is_selected_name(name)) {
			A_gAff.traceEmptyRect(
			ptCoord.x+ptHotSpot.x-4,  
			ptCoord.y+ptHotSpot.y-4, 
			ptCoord.x+ptHotSpot.x+4,  
			ptCoord.y+ptHotSpot.y+4, 
			0);
		}
	}
}



T_BOOL TC_Object::ContientPt(int i_x, int i_y)
{
	if (ptAnimCourante) { // Click sur l'animation
		if (  (ptCoord.x <= i_x)
			&&(i_x       <= ptCoord.x + ptAnimCourante->recRect.right)
			&&(ptCoord.y <= i_y)
			&&(i_y		 <= ptCoord.y + ptAnimCourante->recRect.bottom))
			return T_TRUE;
		else
			return T_FALSE;	
	} else { // click dans une zone transparente
		if (  (ptCoord.x <= i_x)
			&&(i_x       <= ptCoord.x + ptSize.x)
			&&(ptCoord.y <= i_y)
			&&(i_y		 <= ptCoord.y + ptSize.y))
			return T_TRUE;
		else
			return T_FALSE;	
	} 
}




void cgb_obj_warning_resource_update(char *ip_res_name)
{
	int l_cpt;

	for (l_cpt=0; l_cpt<Nb_objetsMax; l_cpt++) {
		if (O_gTab_Objets[l_cpt].isResAnimName(ip_res_name)) {
			O_gTab_Objets[l_cpt].ptAnimCourante = NULL;
			O_gTab_Objets[l_cpt].PtImageAnimCourante = NULL;
			O_gTab_Objets[l_cpt].SetAnimCourantebyName(ip_res_name);
		}
	}
	O_gNb_objets=0;
}
