/*
 *
 *  ==  Coconut ==
 *  2D Adventure Game Engine
 *
 *  Copyright (C) 2000 Sebastien Josset
 *  sebastien.josset@free.fr
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 */

/*
 *
 *  Coconut
 *  Gestion du son
 *
 *
 */
/****** Includes systmes ******/
#include <stdlib.h>

/****** Includes applicatifs ******/
#include "S_son.h"
#include "R_ressources.h"
#include "A_aff.h"


/****** Variables globales ******/
#ifdef USE_SDL
	// CDrom
	SDL_CD *cdrom;

	// Wav
	extern void fill_audio(void *udata, Uint8 *stream, int len);
	static SDL_AudioSpec spec;
	static Uint8 *audio_chunk;
	static SDL_AudioSpec wanted;

	static SDL_AudioCVT cvt[S_channel_NbMax];
	static Uint32		audio_len[S_channel_NbMax];
	static Uint8		*audio_pos[S_channel_NbMax];
	static int			audio_nb[S_channel_NbMax];

#else
	UINT wDeviceID;
	T_DWORD	S_gTimeTrack[100];
#endif

/****** Fonctions de gestion du CDrom Audio ******/

int S_initCD()
{
#ifdef USE_SDL
	cdrom = SDL_CDOpen(0); // CDrom par defaut
	if (!cdrom) {
		fprintf(stderr, "Couldn't open default CD-ROM drive: %s\n",
							SDL_GetError());
	}
#else
	MCI_OPEN_PARMS mciOpenParms;
    MCI_SET_PARMS mciSetParms;
    MCI_STATUS_PARMS mciStatusParms;
    int i, iNumTracks;
    T_DWORD dwReturn;

	// Open the device by specifying the device name.
    mciOpenParms.lpstrDeviceType = "cdaudio";
    if (dwReturn = mciSendCommand(NULL, MCI_OPEN,
        MCI_OPEN_TYPE, (T_DWORD)(LPVOID) &mciOpenParms))
    {
        // Failed to open device. 
        // Don't close device; just return error.
        return 1;
    }

    // The device opened successfully; get the device ID.
    wDeviceID = mciOpenParms.wDeviceID;	

	// Set the time format to minute/second/frame (MSF) format. 
    mciSetParms.dwTimeFormat = MCI_FORMAT_MSF;
    if (dwReturn = mciSendCommand(wDeviceID, MCI_SET, 
        MCI_SET_TIME_FORMAT, 
        (T_DWORD)(LPVOID) &mciSetParms)) 
    {
        mciSendCommand(wDeviceID, MCI_CLOSE, 0, NULL);
        return (dwReturn);
    }

    // Get the number of tracks; 
    // limit to number that can be displayed (20).
    mciStatusParms.dwItem = MCI_STATUS_NUMBER_OF_TRACKS;
    if (dwReturn = mciSendCommand(wDeviceID, MCI_STATUS, 
        MCI_STATUS_ITEM, (T_DWORD)(LPVOID) &mciStatusParms)) 
    {
        mciSendCommand(wDeviceID, MCI_CLOSE, 0, NULL);
        return (dwReturn);
    }
    iNumTracks = mciStatusParms.dwReturn;

	// For each track, get and save the starting location and
	// build a string containing starting locations.
    for(i=1; i<=iNumTracks; i++) 
    {
        mciStatusParms.dwItem = MCI_STATUS_POSITION;
        mciStatusParms.dwTrack = i;
        if (dwReturn = mciSendCommand(wDeviceID, 
            MCI_STATUS, MCI_STATUS_ITEM | MCI_TRACK, 
            (T_DWORD)(LPVOID) &mciStatusParms)) {
            mciSendCommand(wDeviceID, MCI_CLOSE, 0, NULL);
            return (dwReturn);
        }

        S_gTimeTrack[i-1] = mciStatusParms.dwReturn;

		char szTempString[255];
        wsprintf(szTempString, 
            "Track %2d - %02d:%02d:%02d\n", i,
            MCI_MSF_MINUTE(S_gTimeTrack[i-1]), 
            MCI_MSF_SECOND(S_gTimeTrack[i-1]), 
            MCI_MSF_FRAME(S_gTimeTrack[i-1]));
    }

	// Taille totale du CD
	mciStatusParms.dwItem = MCI_STATUS_LENGTH;
    if (dwReturn = mciSendCommand(wDeviceID, 
        MCI_STATUS, MCI_STATUS_ITEM  , 
        (T_DWORD)(LPVOID) &mciStatusParms)) 
	{
		mciSendCommand(wDeviceID, MCI_CLOSE, 0, NULL);
		return (dwReturn);
    }
    S_gTimeTrack[i-1] = mciStatusParms.dwReturn;
	
#endif
	return 0;
}


void S_playCD(long i_track)
{
#ifdef USE_SDL
	if ( CD_INDRIVE(SDL_CDStatus(cdrom)) )
		SDL_CDPlayTracks(cdrom, i_track, 0, 1, 0);
#else
	MCI_PLAY_PARMS	mciPlayParms;

	mciPlayParms.dwCallback = (unsigned long)A_gAff.hwnd;
	mciPlayParms.dwFrom = S_gTimeTrack[i_track];
	mciPlayParms.dwTo  = S_gTimeTrack[i_track+1];
	mciSendCommand(wDeviceID,            // device identifier 
		MCI_PLAY,                        // command message 
		MCI_FROM | MCI_TO ,                               // flags 
		(T_DWORD)(LPVOID) &mciPlayParms);  // parameter block 
#endif

}

void S_stopCD()
{
#ifdef USE_SDL
    SDL_CDStop(cdrom);
#else
	mciSendCommand(wDeviceID, MCI_STOP, 0, NULL);
#endif
}


void S_closeCD()
{
#ifdef USE_SDL
    SDL_CDClose(cdrom);
#else
    mciSendCommand(wDeviceID, MCI_CLOSE, 0, NULL);
#endif
}



/****** Fonctions de gestion des Waves ******/
int S_audio_init()
{
#ifdef USE_SDL
        /* Set the audio format */
        wanted.freq = 22050;
        wanted.format = AUDIO_S16;
        wanted.channels = 2;    /* 1 = mono, 2 = stereo */
        wanted.samples = 1024;  /* Good low-latency value for callback */
        wanted.callback = fill_audio;
        wanted.userdata = NULL;

        /* Open the audio device, forcing the desired format */
        if ( SDL_OpenAudio(&wanted, NULL) < 0 ) {
                fprintf(stderr, "Couldn't open audio: %s\n", SDL_GetError());
                return(-1);
        }
#else
#endif
        return(0);
}


#ifdef USE_SDL

/* The audio function callback takes the following parameters:
    stream:  A pointer to the audio buffer to be filled
    len:     The length (in bytes) of the audio buffer
*/
void fill_audio(void *udata, Uint8 *stream, int len)
{
	int l_cpt;

	// un loop  grer ?
	for (l_cpt=0; l_cpt<S_channel_NbMax; l_cpt++) {
		if ( audio_len[l_cpt] == 0 ) {
			if ( audio_nb[l_cpt] > 0 ) {
				audio_len[l_cpt] = cvt[l_cpt].len_cvt;
				audio_pos[l_cpt] = cvt[l_cpt].buf;
				audio_nb[l_cpt]--;
			}
		}
	}

	// taille max du mix
	for (l_cpt=0; l_cpt<S_channel_NbMax; l_cpt++) {
		if ( audio_len[l_cpt] > 0 ) {
			len = ( (len > audio_len[l_cpt] ) ? audio_len[l_cpt]  : len );
		}
	}
	// mix
	for (l_cpt=0; l_cpt<S_channel_NbMax; l_cpt++) {
        /* Only play if we have data left */
		if ( audio_len[l_cpt] > 0 ) {
			SDL_MixAudio(stream, audio_pos[l_cpt] , len, SDL_MIX_MAXVOLUME);
			audio_pos[l_cpt]  += len;
			audio_len[l_cpt]  -= len;
		}
	}

}


int load_wav(char *ip_fileName, int i_channel, int i_nb)
{
		/* Load the audio data ... */
		if(!SDL_LoadWAV(ip_fileName, &spec, &audio_chunk, &(audio_len[i_channel]))) {
                printf("Couldn't open wav file: %s\n", SDL_GetError());
                return(-1);
        }

		/* convert */
		if(!SDL_BuildAudioCVT(&(cvt[i_channel]), spec.format, spec.channels, spec.freq, 
									wanted.format, wanted.channels, wanted.freq)) {
                printf("Couldn't create convert spec: %s\n", SDL_GetError());
                return(-1);
        }
		cvt[i_channel].len = audio_len[i_channel];
		cvt[i_channel].buf = (Uint8 *)malloc (cvt[i_channel].len*cvt[i_channel].len_mult);
		memcpy(cvt[i_channel].buf, audio_chunk, audio_len[i_channel]);
		//free(audio_chunk);
		if(SDL_ConvertAudio(&(cvt[i_channel]))) {
                printf("Couldn't convert: %s\n", SDL_GetError());
                //return(-1);
        }
		audio_len[i_channel] = cvt[i_channel].len_cvt;
        audio_pos[i_channel] = cvt[i_channel].buf;
		audio_nb[i_channel] = (i_nb-1); // deja jou une fois
		return 0;
}
#endif



void S_playWavFromFile_info(char *ip_fileName, int i_channel, int i_nb)
{
#ifdef USE_SDL
	SDL_PauseAudio(1);
	load_wav(ip_fileName, i_channel, i_nb);
	SDL_PauseAudio(0);
#else
	if ((ip_fileName) && (ip_fileName[0])) {
		PlaySound(ip_fileName, NULL, SND_FILENAME|SND_ASYNC|SND_NODEFAULT  ); 
		//SND_MEMORY
	}
#endif
}


void S_playWavFromFile(char *ip_fileName)
{
	S_playWavFromFile_info(ip_fileName, S_channel_PonctualSound, 1);
}

void S_playWavFromRes(char *ip_resName)
{
	TPC_Res lp_resPtr;
	lp_resPtr = cgb_res_get_ptr_by_name(ip_resName);
	S_playWavFromFile(lp_resPtr->resSon.fileName);
}



