/***************************************************/
/*                                                 */
/* BRAHMS IST Project Demonstrator                 */
/* -------------------------------                 */
/*                                                 */
/* Links Library                                   */
/*                                                 */
/* ll_server_sock.c                                */
/* Server side                                     */
/*                                                 */
/*                              S. Josset          */
/*                              Alcatel Space 2000 */
/*               sebastien.josset@space.alcatel.fr */
/*                                                 */
/***************************************************/

//
// System includes
//
#include <stdio.h>
#include <string.h>
#include <memory.h>
#include <stdlib.h>
#ifndef WIN32
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>                                                      
#endif

//
// Libraries includes
//
#include "ll_sock.h"
#include "port_uxnt.h"

//
// Global variables
//
#define  LL_NbMaxClient 15
int		 LL_FD;						// Listen socket
int		 LL_fd[LL_NbMaxClient];		// Client connexion sockets
TE_LL_id	 LL_fd_id[LL_NbMaxClient];	// Client id
TC_LL_msg LL_g_msg_null;


/***************** Internal functions ********************/
/* return -1: Pb
          >0 :socket
*/
int LL_OpenSocket(int i_port)
{

	struct sockaddr_in 	lv_addr_in;	/* Adresse local AF_INET */
	DWORD   dwValue=5;
 
	/* creation de la socket */
    if ((LL_FD = socket(AF_INET, SOCK_STREAM,0)) < 0) {
      return -1;
    }
    
    /* Construction de l'adresse local */
    lv_addr_in.sin_family = AF_INET;
    lv_addr_in.sin_addr.s_addr = htonl(INADDR_ANY);
    lv_addr_in.sin_port	= htons((unsigned short)i_port);

    /* Creation de lien systeme */
    if (bind((int)LL_FD,
	     (const struct sockaddr *)&lv_addr_in,
	     (int)sizeof(struct sockaddr_in)) < 0) {
      return -1;
    } /* bind */


	/* Ecoute de la socket */
	/* Creation de la liste d'attente (UC_LISTEN_LIMIT appel en attente max) */
	if (listen((int)LL_FD, 1) < 0) {
		printf("Listen Pb\n");
		return -1;
	} /* listen */

	/* Passage en mode non bloquant */
	//P_socketSetNonBlocking(LL_FD);
	P_socketSetNoDelay(LL_FD);
	return LL_FD;
}

int LL_GetFreeSockNum()
{
	int l_cpt;

	/* Le 0 reste inutilis */
	for (l_cpt=1; l_cpt<LL_NbMaxClient; l_cpt++) {
		if (LL_fd[l_cpt]==0) { 
			return l_cpt;
		}
	}
	return -1;
}

int LL_GetNumSockFromId(TE_LL_id i_id)
{
	int l_cpt;

	/* Le 0 reste inutilis */
	for (l_cpt=1; l_cpt<LL_NbMaxClient; l_cpt++) {
		if (LL_fd_id[l_cpt]==i_id) { 
			return l_cpt;
		}
	}
	return -1;
}

void LL_AcceptNewConnection()
{
	/* Acceptation d'une eventuelle connection */
	long		lv_addrsize;	/* Dummy */
	struct		sockaddr_in	lv_addr;	/* Dummy (inutil en AF_UNIX) */
	int			lv_fd;
	int			l_numSock;
	fd_set fdset;
	struct timeval l_tv;
	int l_ret;

	lv_addrsize = sizeof(struct sockaddr_in);
	/* On a encore de la place ? */
	l_numSock = LL_GetFreeSockNum();
	if (l_numSock==-1) {
		return;
	}

	FD_ZERO(&fdset);
	FD_SET((unsigned int)LL_FD, &fdset);
	l_tv.tv_sec = 0;
	l_tv.tv_usec = 0;
	l_ret = select(LL_FD+1, &fdset, NULL, NULL, &l_tv);
	//printf("LL_AcceptNewConnection select=%d\n", l_ret);
	if (  l_ret< 0 ) 
		return;
	if (  l_ret== 0 )
		return;

	if ( FD_ISSET(LL_FD, &fdset) ) {
		lv_fd = accept((int)LL_FD, (struct sockaddr *)&lv_addr, (int *)&lv_addrsize);
		if (lv_fd > 0) {
			/* On ajoute la connexion dans la liste */
			P_socketSetNoDelay(lv_fd);
			//P_socketSetNonBlocking(lv_fd); // blocking socket
			if (LL_fd[l_numSock]) { P_closesocket(LL_fd[l_numSock]); } /* Normalement inutile */
			LL_fd[l_numSock] = lv_fd;
			printf("New connexion accepted\n");
		}
	}
}


/***************** Init      ********************/

int LL_server_init(int i_port)
{
	int l_cpt;
	int l_ret;

    /* init d'une structure vide */
    LL_g_msg_null.num = 0;
    for (l_cpt=0; l_cpt<LL_MSG_BUF_SIZE; l_cpt++) { LL_g_msg_null.buf[l_cpt]=0; }

    /* init */
	P_InitWinsock();
	LL_FD=0;
	for (l_cpt=0; l_cpt<LL_NbMaxClient; l_cpt++) {
		LL_fd[l_cpt]=0;
	}

	l_ret = LL_OpenSocket(i_port);
	return l_ret;
}

/***************** Receive   ********************/
int LL_server_rcv_buf(TC_LL_msg *op_msg)
{
	int l_len;
	int l_sock;

	LL_AcceptNewConnection();

	for (l_sock=0; l_sock<LL_NbMaxClient; l_sock++) {
		if (LL_fd[l_sock]) {
			l_len = P_socketRecv(LL_fd[l_sock], (char *)op_msg, LL_MSG_SIZE, 0);  

			if (l_len==0) {   /* deconnection */
				P_closesocket(LL_fd[l_sock]);
				LL_fd[l_sock] = 0;
				printf("distant deconnected cx(%d)\n", l_sock);
			}
			if (l_len<0)  {   /* nothing to read */
			}
			if (l_len>0) {    /* received something */
				printf("Received Msg [%d octets] by socket[%d] : ",l_len, l_sock);

				(op_msg->magic)		= ntohl(op_msg->magic);
				(op_msg->sender_id) = ntohl(op_msg->sender_id);
				(op_msg->num)		= ntohl(op_msg->num);
				(op_msg->length)	= ntohl(op_msg->length);

				if ((op_msg->magic) != LL_MSG_MAGIC) {
					printf("Pb bad Magic !!!\n");
					return 0;
				}
				LL_fd_id[l_sock] = op_msg->sender_id;

				// Registering msg ??
				if ((op_msg->num==0) && (op_msg->length==0)) {
					printf("Registering Msg from %d\n", op_msg->sender_id);
					return 0;
				} else {
					printf("Msg from %d\n", op_msg->sender_id);
				}
				return 1;
			}
		}
	}
	return 0;
}

/***************** Send      ********************/
int  LL_server_snd_buf(TE_LL_id i_client_id, TC_LL_msg *ip_msg)
{
	int l_sock;
	int l_ret;

	(ip_msg->magic)		= htonl(LL_MSG_MAGIC);
	(ip_msg->sender_id) = htonl(ip_msg->sender_id);
	(ip_msg->num)		= htonl(ip_msg->num);
	(ip_msg->length)	= htonl(ip_msg->length);

	l_ret=0;
	l_sock = LL_GetNumSockFromId(i_client_id);
	if (l_sock==-1) {
		printf("Dest (%d) not connected\n", i_client_id);
	} else {
		if (LL_fd[l_sock]) {
			printf("emission Msg [%d octets] \n", LL_MSG_SIZE);
			P_socketSend(LL_fd[l_sock], (char *)ip_msg, LL_MSG_SIZE, 0);
			l_ret = 1;
		} else {
			printf("Pb, client %d deconnected, socket value=0\n", i_client_id);
		}
	} 

	(ip_msg->magic)		= ntohl(ip_msg->magic);
	(ip_msg->sender_id) = ntohl(ip_msg->sender_id);
	(ip_msg->num)		= ntohl(ip_msg->num);
	(ip_msg->length)	= ntohl(ip_msg->length);

	return l_ret;
}



/***************** Close     ********************/
void LL_server_close()
{
	int l_cpt;
	if (LL_FD) { P_closesocket(LL_FD); }
	for (l_cpt=0; l_cpt<LL_NbMaxClient; l_cpt++) {
		if (LL_fd[l_cpt]) { 
			P_closesocket(LL_fd[l_cpt]); 
			LL_fd[l_cpt]=0;
		}
	}

}

