/***************************************************/
/*                                                 */
/* BRAHMS IST Project Demonstrator                 */
/* -------------------------------                 */
/*                                                 */
/* Port Library                                    */
/*                                                 */
/* port_uxnt.c                                     */
/* Porting functions between Unix/NT               */
/*                                                 */
/*                              S. Josset          */
/*                              Alcatel Space 1999 */
/*               sebastien.josset@space.alcatel.fr */
/*                                                 */
/***************************************************/

//
// System includes
//
#include <stdio.h>

#ifdef WIN32
#include <time.h>
#else
#include <time.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <time.h>
#include <sys/time.h>
#include <sys/times.h>
#include <unistd.h>  
#ifdef __linux__                                                    
#include <asm/ioctls.h>     // Linux                                                   
#endif
#endif

//
// Libraries includes
//
#include "port_uxnt.h"


//
// Global variables
//
int P_g_trace_level=0;
 
/*****************************
/* P_GetTickCount
/*
/* Get time from power on in millisecond
/*****************************/
long P_GetTickCount()
{
#ifdef WIN32
	return GetTickCount();
#else
	/*
	long l_clk;
	long l_ret;
	static long l_clk_per_ms=0;
    struct tms l_tms;
	*/
	// clock() based time
	// Pb under Linux, resolution of 1 s
	// Used under HP.
	//l_clk = clock(); // CPU time used by process
	//l_ret = (l_clk/1000);

	// times() based time
	// times() under Linux return in ms, what we need
	// CLK_TCK=100 ! 
	// CLOCKS_PER_SEC=1000000 !!!???!!! Crazzyy !!!
	//l_clk = times(&l_tms);
	//if (l_clk_per_ms==0) l_clk_per_ms = CLOCKS_PER_SEC/1000;
	//if (l_clk_per_ms==0) l_clk_per_ms = 1000; /* Div 0 parachute */
	//l_ret = l_clk/l_clk_per_ms;
	//l_clk = times(&l_tms);
	//l_ret = l_clk*10;

	//printf("clock(%d) times(%d) clk(%d) persec(%d) perms(%d) ret(%d) sys(%d)\n", 
	//	clock(), l_clk, CLK_TCK, CLOCKS_PER_SEC, l_clk_per_ms, l_ret, l_tms.tms_stime);

	struct timeval now;
	long l_ret;

	gettimeofday(&now, NULL);
	l_ret=(now.tv_sec)*1000+(now.tv_usec)/1000;
	return(l_ret);


	//return l_ret;
#endif
}


/*****************************
/* P_Sleep
/*
/* Sleep l_ms millisecond
/*****************************/
void P_Sleep(long l_ms)
{
#ifdef WIN32
	Sleep((DWORD)l_ms);
#else
	// select timeout method
	struct timeval l_tv;
	int l_ret;

    l_tv.tv_sec = l_ms/1000;
    l_tv.tv_usec = (l_ms-(l_tv.tv_sec*1000))*1000;
	l_ret = select(0, NULL, NULL, NULL, &l_tv);

/*
	// While method: really bad...CPU consuming...but it works
	long l_tick_deb;
    l_tick_deb = P_GetTickCount();
	while ((P_GetTickCount()-l_tick_deb)<l_ms) { }
*/	
#endif
}


/*****************************
/* P_InitWinsock
/*
/* Initialise the sockets under Windows
/*****************************/
void P_InitWinsock()
{
#ifdef WIN32
	/* Init des winsock*/
	WORD wVersionRequested;
	WSADATA wsaData;
	int err;
	wVersionRequested = MAKEWORD( 2, 0 ); 
	err = WSAStartup( wVersionRequested, &wsaData );
	if ( err != 0 ) {
		/* Tell the user that we couldn't find a usable */
		/* WinSock DLL.                                  */
		return;
	}
#endif
}


/*****************************
/* P_closesocket
/*
/* Close a socket
/*****************************/
void P_closesocket(int i_FD)
{
#ifdef WIN32
	closesocket(i_FD);
#else
	close(i_FD);
#endif
}


/*****************************
/* P_socketSetNoDelay
/* 
/* Disable the Nagle algorithme.
/* TCP buffer are directly sent, instead of
/* beeing bufferized
/*****************************/
void P_socketSetNoDelay(int i_fd)
{
#ifdef WIN32
	int	l_val;
	int l_ret;
	struct linger l_linger;
 
	l_val = 1;
	l_linger.l_onoff = 1;
	l_linger.l_linger= 0;

	/* Disable Nagle */
	if (setsockopt ( i_fd, IPPROTO_TCP, TCP_NODELAY, (const char *)&l_val, sizeof(int) )) {
		printf("Socket buffering not desactivated\n");
	}
	 
	/* Allow fast close */
	if (setsockopt ( i_fd, SOL_SOCKET , SO_LINGER, (const char *)&l_linger, sizeof(struct linger) )) {
		printf("SO_LINGER not activated\n");
		l_ret=WSAGetLastError ();
		switch (l_ret){

 		case WSANOTINITIALISED:
			printf("A successful WSAStartup must occur before using this function. \n");
			break;
 		case WSAENETDOWN:
			printf("The network subsystem has failed. \n");
			break;
 		case WSAEFAULT:
			printf("optval is not in a valid part of the process address space or optlen parameter is too small. \n");
			break;
 		case WSAEINPROGRESS:
			printf("A blocking Windows Sockets 1.1 call is in progress, or the service provider is still processing a callback function. \n");
			break;
		case WSAEINVAL:
			printf("level is not valid, or the information in optval is not valid.\n");
			break;
		case WSAENETRESET:
			printf("Connection has timed out when SO_KEEPALIVE is set.\n");
			break;
		case WSAENOPROTOOPT:
			printf("The option is unknown or unsupported for the specified provider.\n");
			break;
		case WSAENOTCONN:
			printf("Connection has been reset when SO_KEEPALIVE is set.\n");
			break;
		case WSAENOTSOCK:
			printf("The descriptor is not a socket.\n");
			break;
		default: 
			break;
		}
	}

#else
	fcntl((int) i_fd, F_SETFL, O_NDELAY);	 
#endif
}

/*****************************
/* P_socketSetNonBlocking
/* 
/* Socket is put in non-blocking mode
/* Tips: Use select instead of this...
/*****************************/
void P_socketSetNonBlocking(int i_fd)
{
#ifdef WIN32
	/* Passage en mode non bloquant */
	unsigned long l_param;
 
	l_param = 1;
	ioctlsocket (i_fd, FIONBIO ,&l_param ); 

#else
	fcntl((int) i_fd,F_SETFL,O_NONBLOCK);	
#endif
}



/*****************************
/* P_AffWorking
/* 
/* Print a spinning wheel as an activity indicator:
/* e.g.: |  /  -  \  |
/* i_cpt: a counter of avancement
/*****************************/
void P_AffWorking(int i_cpt)
{
	switch (i_cpt%4) {
#ifdef WIN32
	case 0: 	printf("|%c",8); break;
	case 1: 	printf("/%c",8); break;
	case 2: 	printf("-%c",8); break;
	case 3: 	printf("\\%c",8); break;
#else
	case 0:
		printf("|");
		fflush(stdout);
		printf("%c",8);
		fflush(stdout);
		break;
	case 1:
		printf("/");
		fflush(stdout);
		printf("%c",8);
		fflush(stdout);
		break;
	case 2:
		printf("-");
		fflush(stdout);
		printf("%c",8);
		fflush(stdout);
		break;
	case 3:
		printf("\\");
		fflush(stdout);
		printf("%c",8);
		fflush(stdout);
		break;
#endif
	default: 	printf("."); break;
	}
}

/******************************************/
/* P_socketSend
/* 
/* Send a buffer to a socket.
/* If the buffer an't be send, the function wait
/* and retry until it's successfull.
/* Return >0: Nb bytes sent
/*         0: Nothing sent, Pb somewhere
/*		  -1: Pb
/******************************************/
int P_socketSend(int i_fd, char *ip_buf, int i_len, int i_flags)
{
	int l_ret;
	int l_waiting_time=1;

	if (!i_fd) { 
		printf("P_socketSend fd=NULL\n");
		return 0;
	}
	if (!ip_buf) { 
		printf("P_socketSend ip_buf=NULL\n");
		return 0;
	}
	if (!i_len) { 
		printf("P_socketSend i_len=0\n");
		return 0;
	}

P_socketSend_try:
	l_ret = send(i_fd, ip_buf, i_len, i_flags);
	if ((l_ret>0)&&(l_ret != i_len)) { printf("P_socketSend: Pb Partial send ****\n"); } 

	P_traceBuf(T_INFO, 0,  ip_buf, "Envoi Socket",l_ret);
#ifdef WIN32
	if (l_ret<0) {
		printf("WSAGetLastError()==%d\n",WSAGetLastError() );
		if (WSAGetLastError()==WSAEWOULDBLOCK) {
			printf("No send buffer available at  network layer. Retry in %d ms\n", l_waiting_time);
			P_Sleep(l_waiting_time);
			if (l_waiting_time<3000) l_waiting_time = 2*l_waiting_time;
			goto P_socketSend_try;
		}
	}
#else
	if (l_ret<0) {
		printf("send()==%d\n",l_ret );
		if ((errno==EMSGSIZE)||(errno==EWOULDBLOCK)||(errno==ENOBUFS)) {
			printf("No send buffer available at  network layer. Retry in %d ms\n", l_waiting_time);
			P_Sleep(l_waiting_time);
			if (l_waiting_time<3000) l_waiting_time = 2*l_waiting_time;
			goto P_socketSend_try;
		}
	}

#endif
	//if (l_ret>=0) { l_ret=i_len; } 
	return l_ret;
}



/************************************/
/* P_socketRecv
/* Receive data on from a socket
/* Return : >0: Nb bytes received
/*           0: Cx closed
/*          -1: Nothing to read
/************************************/
int P_socketRecv ( int i_fd,char *op_buf, int i_len, int i_flags )
{
	fd_set fdset;
	struct timeval l_tv;
	int l_ret;
	int l_available;
	int l_len;

	//printf("Receive data on socket %d ?\n", i_fd);
	/* Wait for data */
	FD_ZERO(&fdset);
	FD_SET((unsigned int)i_fd, &fdset);
	l_tv.tv_sec = 0;
	l_tv.tv_usec = 0;
	l_ret = select(i_fd+1, &fdset, NULL, NULL, &l_tv);
	//printf("P_socketRecv select=%d\n", l_ret);
	if (  l_ret< 0 ) 
		return -1;
	if (  l_ret== 0 )
		return -1;

	if ( FD_ISSET(i_fd, &fdset) ) {

		/* Get available data size */
#ifdef WIN32
		ioctlsocket( i_fd, FIONREAD, &l_available );
#else
		ioctl( i_fd, FIONREAD, &l_available );
#endif
		printf("P_socketRecv available=%d\n", l_available);
		if ((l_available>0)&&(l_available<i_len)) {
			printf("Wait for more\n");
			return -1;
		}

		l_len = i_len;
P_socketRecv_try:
		l_ret = recv ( i_fd, op_buf, l_len, 0); // assume blocking recv
		printf("P_socketRecv waited=%d recv=%d\n", i_len, l_ret);
		if ( l_ret <= 0 )
			return 0;  /* End of transmission */
		if (l_ret<l_len) {
			op_buf += l_ret;
			l_len -= l_ret;
			goto P_socketRecv_try; // should never be used (non blocking socket)
		}
		return i_len;
	}

	return -1;
}


/************************************/
/* P_get_date
/* Return the date, in a printable form
/* iop_date : buffer to contain date string
/************************************/
void P_get_date(char *	iop_date)
{
    int                                 lv_len;
    static time_t				lv_time_sec;	/* date in secondes */
  #ifndef WIN32
    static struct timeval		lv_time_val;	/* date in secondes & millis */
    static struct timezone		lv_time_zone;
    static struct tm			*lv_time_local;
  #endif

	time(&lv_time_sec);
  #ifndef WIN32 /**** ms are not available under Windows ***/
    gettimeofday(&lv_time_val, &lv_time_zone);
    lv_time_local = localtime(&lv_time_sec);     
    sprintf(iop_date, "%02d/%02d/%02d %02d:%02d:%02d:%06d",
		 lv_time_local->tm_mday, (lv_time_local->tm_mon)+1,
		 lv_time_local->tm_year, lv_time_local->tm_hour,
		 lv_time_local->tm_min, lv_time_local->tm_sec,
		 lv_time_val.tv_usec);
  #else
    strcpy(iop_date, ctime(&lv_time_sec));
  #endif
  lv_len=strlen(iop_date)-1;
  if(iop_date[lv_len]=='\n') iop_date[lv_len]=0;   
}


/************************************/
/* P_get_entete_trace
/* Return the trace header, in a printable form
/* OUT:
/* iop_date : buffer to contain the trace header*
/* IN:
/* ip_sourceFilename : source file name to be added in header
/* i_sourceLine		 : source line to be added in header
/************************************/
void P_get_entete_trace(
		  char *		iop_entete, 
		  const char *	ip_sourceFilename, 
		  int	     	i_sourceLine)
{
    static char lv_date[64];
    char * lv_ptr_sourceFilename;

    /**** Get name without path ***/
     if(!(lv_ptr_sourceFilename=strrchr(ip_sourceFilename,'/')))
       lv_ptr_sourceFilename=(char *)ip_sourceFilename;
     else
     if(!(lv_ptr_sourceFilename=strrchr(ip_sourceFilename,'\\')))
       lv_ptr_sourceFilename=(char *)ip_sourceFilename;
     else
       lv_ptr_sourceFilename++; /** jump over the / **/

    /**** Get date in s & ms ****/
	P_get_date(lv_date);
	sprintf(iop_entete,"[%25s, %4d]<%s>",
		lv_ptr_sourceFilename, i_sourceLine,
		lv_date);

}

/************************************/
/* P_traceBuf
/* Dump a buffer trace
/* IN:
/* i_sourceLine			: source line to be added in header
/* ip_sourceFilename	: source file name to be added in header
/* i_seuil_trace		: trace level
/* *ip_buf				: buffer to dump
/* *ip_label			: title of the dump
/* i_taillebuf			: size of buf to dump
/************************************/
void P_traceBuf(int i_sourceLine,
		      char *ip_sourceFilename,
		      int  i_seuil_trace,
		      char *ip_buf,
		      char *ip_label,
		      int  i_taillebuf)
{
    static  int	 lv_size;
    static  char lv_char[] = "0123456789ABCDEF";
    static  char lv_message[1024];
    static  int	 lv_position;
    static  char *lv_octet_ptr;
    static  int	 lv_cpt1;
    static  int  lv_taillebuf;

    const   int  UD_DUMP_OFFSET_SIZE=	7;
    const   int  UD_DUMP_LINE_SIZE=	74;
    const   int  UD_DUMP_SEP_POS=		55;
    const	int  UD_DUMP_ASC_POS=	57;
    const	int  UD_BYTESPERLINE=	 16;

	if (P_g_trace_level==0) return;

    lv_taillebuf=i_taillebuf;


	if ((i_taillebuf<0)) { 
		printf("Trace Error : taille buf <0\n");
		return;
	}
	if ((i_taillebuf>100)) { 
		printf("Trace taille buf %d>100, dump limite a 32\n", i_taillebuf);
		return;
	}

     /**** get entete */
      P_get_entete_trace(lv_message,ip_sourceFilename, i_sourceLine);
      lv_size=strlen(lv_message);

      /**** raw buffer */
      lv_size+=sprintf (&lv_message[lv_size]," : Buffer lg= %d (0x%x) : %s", lv_taillebuf, lv_taillebuf, ip_label);	 
      lv_message[lv_size] = '\n';
      lv_message[lv_size+1] = 0;
      
      /* dump in file */
	  /*
      if (fwrite(lv_message, lv_size+1, 1, l_fh) != 1)
			return;
	  */
      /* dump at screen  */
      printf(lv_message);


      /**** hexa dump */
      if (lv_taillebuf != 0)
      {
		 for (lv_cpt1=0,
			  lv_octet_ptr = ip_buf;
			  lv_taillebuf; lv_taillebuf--, lv_cpt1++, lv_octet_ptr++)
		 {
			/* built a line */
			if ((lv_position = lv_cpt1 % UD_BYTESPERLINE) == 0)
			{
			   /* built an empty line */
			   sprintf(lv_message,"%05x: ",lv_cpt1);
			   memset(&lv_message[UD_DUMP_OFFSET_SIZE],' ',UD_DUMP_LINE_SIZE);
			   lv_message[UD_DUMP_SEP_POS] = '|';
			   lv_message[UD_DUMP_LINE_SIZE] = '\n';
			   lv_message[UD_DUMP_LINE_SIZE+1] = '\0'; 
			}
			
			lv_message[lv_position*3+UD_DUMP_OFFSET_SIZE] =
			   lv_char[(*lv_octet_ptr >> 4) & 0x0F];
			lv_message[lv_position*3+UD_DUMP_OFFSET_SIZE+1] =
			   lv_char[(*lv_octet_ptr) & 0x0F];
			if (isprint(*lv_octet_ptr))
			   lv_message[UD_DUMP_ASC_POS+lv_position] = (*lv_octet_ptr);
			else
			   lv_message[UD_DUMP_ASC_POS+lv_position] = '.';
			
			/* Test: Size of line */
			if (lv_position == UD_BYTESPERLINE - 1)
			{
			   /* Write the line */
				/*
			   if (fwrite(lv_message, UD_DUMP_LINE_SIZE+1, 1, iop_contexte->fh) != 1)
					return;		
				*/
			   /* Write the line */
				lv_message[UD_DUMP_LINE_SIZE+1]=0;
			   printf(lv_message);
			} /* if */
		 } /* for */
		 
		 if (lv_position != UD_BYTESPERLINE - 1)
		 {
			   /* Write the line */
			 /*
			if (fwrite(lv_message, UD_DUMP_LINE_SIZE+1, 1, iop_contexte->fh) != 1)
				return;  
			*/
			lv_message[UD_DUMP_LINE_SIZE+1]=0;
			printf(lv_message);
		 }
      }
 
	  /***** update trace context ****/
	  /*
	  iop_contexte->nb_ecritures++;
	  if (iop_contexte->nb_ecritures>iop_contexte->frequence_scan)
		ud_check_size_file_trace(iop_contexte);
	  */
/*	} */
}
